/*
One of those Weeks episode 35:BSoDs in Space
Copyright (c) 2008-2009 Blazing Games Inc.
Code written by Billy D. Spelchan

This program is free software; you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software 
Foundation; either version 2 of the License, or (at your option) any later 
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.ootw.ep35
{
	// Adobe Imports
//	import flash.display.SimpleButton;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.geom.Rectangle;
//	import flash.events.MouseEvent;
//	import flash.text.TextField;
	import flash.ui.Keyboard;
	import flash.utils.*;
	
	public class OotW35
	extends MovieClip
	{
// ***** CONSTANTS ***********************************************************

		public static const DEATH_EVENT:String = "DEATH Event";
		public static const WIN_EVENT:String = "WIN Event";

		protected static const FIRERATE:int = 250; // in ms
		protected static const MAXLASERS:int = 11;
		protected static const MAXBSODS:int = 64 * 6;

		protected static const BSOD_SPAWN_X:Array = 
				new Array (100, 320, 540, 100, 320, 540);
		protected static const BSOD_SPAWN_Y:Array = 
				new Array (100, 380, 100, 380, 100, 380);
				
// ***** VARIABLES ***********************************************************

		protected var _lastRedraw:int;
		protected var _turningLeft:Boolean;
		protected var _turningRight:Boolean;
		protected var _accelerating:Boolean
		protected var _firing:Boolean;

		protected var _bsods:Array;
		protected var _BSoDIndex:int;

		protected var _lasers:Array;
		protected var _lastShotTime:int;
		protected var _lastLaserIndex:int;

		protected var _score:int;
		protected var _gameOver:Boolean;

		// note that all _*_movie, _*_btn and _*_txt variables are in movie

// ***** CONSTRUCTOR *********************************************************

		public function OotW35()
		{
			var cntr:int;
			
			_lastRedraw = getTimer();
			_lastShotTime = _lastRedraw;
			_lastLaserIndex = 0;
			stage.addEventListener(KeyboardEvent.KEY_DOWN, handleKeyDown); 
			stage.addEventListener(KeyboardEvent.KEY_UP, handleKeyUp); 
			stage.addEventListener(Event.ENTER_FRAME, handleFrame);
			// bsod layer
			_bsods = new Array();
			for (cntr = 0; cntr < MAXBSODS; ++cntr) {
				_bsods[cntr] = new BSoD();
				addChild(_bsods[cntr]);
			}
			restartGame(6);

			// bolt layer
			_lasers = new Array(MAXLASERS);
			for (cntr = 0; cntr < MAXLASERS; ++cntr) {
				_lasers[cntr] = new LaserBolt();
				addChild(_lasers[cntr]);
			}
		}

// ***** Functions - Event handlers ******************************************

		public function handleKeyDown(evt:KeyboardEvent)
		{
			var kc:uint = evt.keyCode;
			
			switch (kc) {
				case Keyboard.LEFT:
					_turningLeft = true;
					break;
				case Keyboard.RIGHT:
					_turningRight = true;
					break;
				case Keyboard.UP:
					_accelerating = true;
					break;
				case Keyboard.SPACE:
					_firing = true;
					break;
			}
		}

		public function handleKeyUp(evt:KeyboardEvent)
		{
			var kc:uint = evt.keyCode;
			
			switch (kc) {
				case Keyboard.LEFT:
					_turningLeft = false;
					break;
				case Keyboard.RIGHT:
					_turningRight = false;
					break;
				case Keyboard.UP:
					_accelerating = false;
					break;
				case Keyboard.SPACE:
					_firing = false;
					break;
			}
		}

		public function handleFrame(evt:Event)
		{
			if (_gameOver)
				return;
			var curtime:int = getTimer();
			var delta:Number = Math.min(1.0, 
					Number(curtime - _lastRedraw)/1000);
			var cntr:int;
			var bsodID:Number;

			_lastRedraw = curtime;
			if (_turningLeft)
				_ship_movie.turnLeft(delta);
			if (_turningRight)
				_ship_movie.turnRight(delta);
			if (_accelerating)
				_ship_movie.accelerate(delta);
			if ((_firing) && ((curtime - _lastShotTime) > FIRERATE)){
				_lasers[_lastLaserIndex].launch(_ship_movie);
				++_lastLaserIndex;
				if (_lastLaserIndex >= MAXLASERS)
					_lastLaserIndex = 0;
				_lastShotTime = curtime;
			}
			_ship_movie.updatePosition(delta);
			
			if (checkShipCollision() > -1) {
				_gameOver = true;
				var evnt:Event = new Event(DEATH_EVENT);
				this.dispatchEvent(evnt);
			}
			
			for (cntr = 0; cntr < MAXLASERS; ++cntr) {
				if (_lasers[cntr].visible) {
					_lasers[cntr].updatePosition(delta);
					bsodID = checkBSoDCollision(_lasers[cntr]);
					if (bsodID > -1) {
						explodeBSoD(bsodID);
						_lasers[cntr].visible = false;
					}
				}
			}
			for (cntr = 0; cntr < _BSoDIndex; ++cntr)
				_bsods[cntr].updatePosition(delta);
		}

// ***** Functions - General control *****************************************

		public function restartGame(numBigBSoDs:int)
		{
			var cntr:int;
			for (cntr = 0; cntr < MAXBSODS; ++cntr) {
				_bsods[cntr].visible = false;
			}
			
			for (cntr = 0; cntr < numBigBSoDs; ++cntr) 
				_bsods[cntr].spawn(BSOD_SPAWN_X[cntr],BSOD_SPAWN_Y[cntr],0);
			_BSoDIndex = numBigBSoDs;
			_score = 0;
			_gameOver = false;
			_ship_movie.x = 320;
			_ship_movie.y = 240;
			_ship_movie.fullStop();
		}

		public function checkBSoDCollision(mc:MovieClip):int
		{
			var bsodHit:int = -1;
			var dist:Number, deltaX:Number, deltaY:Number;
			var halfWidth:Number, hwSquared:Number;
			var cntr:int;

			for (cntr = 0; cntr < _BSoDIndex; ++cntr) {
				if ( ! _bsods[cntr].visible)
					continue;
					
				if (_bsods[cntr].hitTestObject(mc))
					bsodHit = cntr;
/*
				halfWidth = _bsods[cntr].width / 2 + mc.width / 2;
				hwSquared = halfWidth * halfWidth;
				deltaX = _bsods[cntr].x - mc.x;
				deltaY = _bsods[cntr].y - mc.y;
				dist = deltaX * deltaX + deltaY * deltaY
				if (hwSquared > dist)
					bsodHit = cntr;
*/
			}
			return bsodHit;
		}

		public function checkShipCollision():int
		{
			var cntr:int;
			var bsodHit:int = -1;
			var bsodBounds:Rectangle;
			var shipBounds:Rectangle = _ship_movie.getBounds(this);

			shipBounds.x += (shipBounds.width * 0.15);
			shipBounds.y += (shipBounds.height * 0.15);
			shipBounds.width = (shipBounds.width * 0.7);
			shipBounds.height = (shipBounds.height * 0.7);
			for (cntr = 0; cntr < _BSoDIndex; ++cntr) {
				if ( ! _bsods[cntr].visible)
					continue;
				bsodBounds = _bsods[cntr].getBounds(this);
				bsodBounds.x += (bsodBounds.width * 0.2);
				bsodBounds.y += (bsodBounds.height * 0.2);
				bsodBounds.width = (bsodBounds.width * 0.6);
				bsodBounds.height = (bsodBounds.height * 0.6);
				
				if (bsodBounds.intersects(shipBounds)) {
					trace ("hit bsod " + bsodBounds + " you " + shipBounds);
					bsodHit = cntr;
				}
			}
			return bsodHit;
		}

		public function explodeBSoD(n:int)
		{
			++_score;
			var gen:int = _bsods[n].getGeneration() + 1;
			if (gen > 3) {
				trace("Killing of bsod here");
				_bsods[n].visible = false;
				// check to see if game over
				var bsodCount:int = 0;
				for (var cntr:int = 0; cntr < MAXBSODS; ++cntr)
					if (_bsods[cntr].visible)
						++bsodCount;
				if ( bsodCount == 0 ) {
					_gameOver = true;
					var evnt:Event = new Event(WIN_EVENT);
					this.dispatchEvent(evnt);
				}
			} else {
				// Exploded bsods start where other bsod was
				var bx:Number = _bsods[n].x;
				var by:Number = _bsods[n].y;
				var bw:Number = _bsods[n].width; /* _bsods[n].scaleX;*/
				var bh:Number = _bsods[n].height; /* _bsods[n].scaleY;*/
				
				var bx1:Number = bx - bw / 4;
				var bx2:Number = bx + bw / 4;
				var by1:Number = by - bh / 4;
				var by2:Number = by + bh / 4;
				
				_bsods[n].spawn(bx1, by1, gen);
				_bsods[_BSoDIndex].spawn(bx2, by1, gen);
				++_BSoDIndex;
				_bsods[_BSoDIndex].spawn(bx1, by2, gen);
				++_BSoDIndex;
				_bsods[_BSoDIndex].spawn(bx2, by2, gen);
				++_BSoDIndex;
			}
		}
		
		public function getScore():int
		{
			return _score;
		}
	} // end class
} // end package

